# websms

This is the "websms" Module. It provides possibility to send text messages via https://websms.at/sms-gateway-api 

# Important notice

## Phone numbers (List of recipients)

The sms gateway offers worldwide delivery and therefore the phone numbers in the list of recipients need to be formatted as E164 MSISDNs (international format) to allow proper message routing. Hence, numbers are limited to a maximum of 15 digits, including the international call prefix (e.g. 431234567890000).

see Wikipedia https://https://en.wikipedia.org/wiki/E.164 https://en.wikipedia.org/wiki/MSISDN 


## SMS encoding

sms messages have a very limited space, which is always 140 Bytes of data. The websms backend will automatically generate multiple sms and concatenate them together, if the transmitted text does not fit in one sms. The amount of generated sms is capped by the configuration parameter "maxSmsPerMessage" where an error is returned if exceeded and highly depends on the submitted characters.

Although the message content (text) is transferred in UTF-8 to the server, the message content is encoded on the server side to either

- GSM 7-bit default alphabet
- UCS-2 Encoding (matches UTF-16 encoding)

which has an impact on the maximum amount of characters which fit into one sms part. This can lead to higher costs than expected.

The standard encoding for GSM messages is the 7-bit default alphabet. When only characters are in the message which can be mappted to this encoding, it is possible to send up to 160 characters (packed in up to 140 octets) in one SMS message in the GSM network. Additional, the GSM characters €, [, ], {, }, |, ^, ~, and \ are located in an extension table and consume the space of two characters.

Hence, a message which contains characters that are not defined in the GSM 7-bit default alphabet will convert to UCS-2 encoding and as UCS-2 is a 16bit alphabet a single SMS can have at most 70 characters (in 140 octets).

If more than 160 or 70 characters, depending on the encoding, are submitted and the server will automatically generate concatenated sms, the concatenation will use additional metadata, which will reduce the amount of characters in GSM encoding to 153 and in case of UCS-2 encoding to 67 characters.

As the maxSmsPerMessage defaults to 3, a maximum of 3 sms will be generated in the default setup.


# Installation

After extracting the extensions add the following to the localextensions.xml

```
<!-- websms -->
<extension name="websmscore"/>
<extension name="websmsfacades"/>
<extension name="websmsbackoffice"/>
```

## Used typecodes
- 14008 -> WebSmsConfiguration

# Usage

## Configuration

First a configuration for sending sms has to be created. Go to the backoffice `websms` -> `Configuration` and add a new configuration. Enter the following information:

- Code: String which describes the configuration
- Token: The login token obtained from websms account
- Max SMS per message: Default value how many sms per message should be send. Can be overridden `WebsmsRestWSRequest#maxSmsPerMessage` on each request

### Test configuration

After saving the configuration you can select it in the list and configuration and in the header of the list view you see an icon for `Test configuration`. After clicking that icon, a window will open enter the following information:

- Phone number: in international format (e.g. 431234567890000)
- Message: The message to send
- Test: When selected `true` the transmission is only simulated; no SMS is sent. Select `false` the SMS is sent via the SMS Gateway.

## Integrate into project

For sending sms in the project use the `WebsmsTextMessageApiFacade` interface. Which is implemented in the class `DefaultWebsmsTextMessageApiFacade`.

WebsmsRestWSRequest has 2 mandatory parameters
- recipientAddressList: The list of phone numbers in international format
- messageContent: The text to sent to the defined recipients


For more information see the java doc and https://developer.websms.com/rest-api/smsmessaging_text_post/#TextSmsSendRequest.

## Connection to API

The connection URL can be configured via property `websms.api.url` which defaults to `https://api.websms.com/rest/smsmessaging/text`. It needs to be ensured that this API endpoint is reachable from network.

For establishing the connection and sending the request `org.springframework.web.client.RestTemplate` class is used. Which is initialized using the default constructur. It is possible to overwrite the creation in `DefaultWebsmsTextMessageApiFacade#getRestTemplate()` method.